"""
Module realize Luna-Remote-SDK mixin
"""

from typing import Awaitable, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub
from luna3.public.sdk import VideoSDKInput


class LVAMixin(MixinStub):
    """Luna-Video-Agent Mixin to work with videos."""

    def videosdk(
        self, inputData: VideoSDKInput, accept: str = "application/msgpack", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Process video

        Args:
            inputData: video properties including url and analytics params

        Returns:
            LunaResponse: body contains json with analytics results
            errors for failed images.


        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.public.sdk import VideoSDKInput, VideoSDKAnalytics, VideoSDKProperties
        >>> from luna3.public.sdk import PeopleCountAnalyticsParams, Rate
        >>> url ="https://lunatestttttt.s3.eu-central-1.amazonaws.com/video_test.avi"
        >>> params = PeopleCountAnalyticsParams(rate=Rate(unit="second", period=1), probeCount=2)
        >>> analytics = [VideoSDKAnalytics(name="people_count", targets=[], parameters=params)]
        >>> response = client.videosdk(inputData=VideoSDKInput(VideoSDKProperties(url), analytics))
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "analytics": [
                {
                    "name": "people_count",
                    "result": {
                        "crowd": {
                            "events": [
                                {
                                    "event_id": ...,
                                    "frames_estimations": [
                                        {
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "people_count": ...,
                                            "time_offset": ...
                                        }
                                    ],
                                    "people_count": 2,
                                    "track_id": ...,
                                    "video_segment": {
                                        "end_time_offset": ...,
                                        "start_time_offset": 1.0
                                    }
                                }
                            ]
                        }
                    }
                }
            ],
            "video_data": {
                "duration": 5.0,
                "error_count": 0,
                "frame_rate": 24.0,
                "frames_count": 120,
                "processed_parts": [
                    {
                        "end_offset": ...,
                        "start_offset": 0
                    }
                ],
                "rotation": ...
            }
        }
        """
        payload = RequestPayload.buildJson(inputData.asDict())
        return self.makeRequest(f"{self.baseUri}/videosdk", "POST", body=payload, accept=accept, **kwargs)

    def getVideoTask(self, taskId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get video task data

        Args:
            taskId: task id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with task status, progress or result url.


        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.public.sdk import VideoSDKInput,VideoSDKAnalytics,VideoSDKProperties,DeferredTask
        >>> from luna3.public.sdk import PeopleCountAnalyticsParams, Rate, StoragePolicy
        >>> url ="https://lunatestttttt.s3.eu-central-1.amazonaws.com/video_test.avi"
        >>> params = PeopleCountAnalyticsParams(rate=Rate(unit="second", period=1), probeCount=2)
        >>> analytics = [VideoSDKAnalytics(name="people_count", targets=[], parameters=params)]
        >>> deferred = DeferredTask(1, StoragePolicy("luna_image_store"))
        >>> response = client.videosdk(inputData=VideoSDKInput(VideoSDKProperties(url), analytics, deferred))
        >>> taskId = response.json["task_id"]  # doctest: +ELLIPSIS
        >>> response = client.getVideoTask(taskId)
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "progress": null,
            "status": ...,
            "task_id": ...
        }
        """
        return self.makeRequest(f"{self.baseUri}/video/tasks/{taskId}", "GET", **kwargs)
