"""Some structures for Streams Api."""

from dataclasses import asdict, dataclass
from typing import List, Literal, Optional, Union

from stringcase import snakecase

from luna3.common.http_objs import TaskCallback, TelegramCallback


@dataclass
class StreamInfo:
    """Stream log info"""

    streamId: str
    status: int
    version: int
    time: str
    analyticIndexes: Optional[List[int]]
    lastProcessedFrameTime: float = None
    error: Optional[str] = None

    def asDict(self):
        """Creates dict representation."""
        return {
            "stream_id": self.streamId,
            "status": self.status,
            "error": self.error,
            "version": self.version,
            "time": self.time,
            "last_processed_frame_time": self.lastProcessedFrameTime,
            "analytic_indexes": self.analyticIndexes,
        }


@dataclass
class GeoPosition:
    """Geo position class"""

    latitude: float
    longitude: float


@dataclass
class Location:
    """Location structure."""

    area: Optional[str] = None
    city: Optional[str] = None
    district: Optional[str] = None
    street: Optional[str] = None
    houseNumber: Optional[str] = None
    geoPosition: Optional[GeoPosition] = None

    def asDict(self):
        """Creates dict representation."""
        return {snakecase(field): value for field, value in asdict(self).items() if value is not None}


@dataclass
class Restart:
    """Restart configuration structure."""

    restart: Optional[bool] = False
    attemptCount: Optional[int] = 10
    delay: Optional[int] = 60

    def asDict(self):
        """Creates dict representation."""
        return {snakecase(field): int(value) for field, value in asdict(self).items() if value is not None}


@dataclass
class Agent:
    """Video manager agent structure"""

    name: str
    maxStreamCount: int
    port: int
    apiVersion: int
    analyticNames: List[str]
    description: Optional[str] = None
    host: Optional[str] = None

    def asDict(self) -> dict:
        """Creates dict representation."""
        res = {
            "name": self.name,
            "port": self.port,
            "api_version": self.apiVersion,
            "max_stream_count": self.maxStreamCount,
            "analytic_names": self.analyticNames,
        }
        if self.description is not None:
            res["description"] = self.description
        if self.host is not None:
            res["host"] = self.host
        return res


@dataclass
class Analytic:
    """Video manager analytic structure"""

    analyticName: str
    version: Optional[int] = None
    description: Optional[str] = None
    documentation: Optional[str] = None
    defaultParameters: Optional[dict] = None
    validationSchema: Optional[dict] = None
    host: Optional[str] = None

    def asDict(self) -> dict:
        """Creates dict representation"""
        res = {"analytic_name": self.analyticName}
        if self.version is not None:
            res["analytic_version"] = self.version
        if self.description is not None:
            res["description"] = self.description
        if self.documentation is not None:
            res["documentation"] = self.documentation
        if self.defaultParameters is not None:
            res["default_parameters"] = self.defaultParameters
        if self.validationSchema is not None:
            res["validation_schema"] = self.validationSchema
        if self.host is not None:
            res["host"] = self.host
        return res


@dataclass
class StreamAnalytic:
    """Video manager stream analytic"""

    name: str
    parameters: Optional[dict] = None

    def asDict(self) -> dict:
        """Create dictionary representation"""
        res = {"analytic_name": self.name}
        if self.parameters is not None:
            res["parameters"] = self.parameters
        return res


@dataclass
class StreamNotificationPolicy:
    """
    Stream notification policy
    """

    enable: Literal[0, 1]
    streamCallbacks: List[Union[TaskCallback, TelegramCallback]]

    def asDict(self) -> dict:
        """Prepare dict representation."""
        return {
            "enable": self.enable,
            "stream_callbacks": [taskCallback.asDict() for taskCallback in self.streamCallbacks],
        }
